/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#include "systemc.h"
#include <iostream>
#include <fstream>
#include <string>
#include "boost/program_options.hpp"

#include "utils/GlobalResourcesCS.h"
#include "networkManager/NetworkManager.h"
#include "utils/report.h"

namespace po = boost::program_options;

int sc_main(int arg_num, char *arg_vec[])
{
    po::variables_map vm;
    po::options_description desc("Allowed Options");
    desc.add_options()("configFolder", po::value<std::string>()->default_value(""), "Input path for config.xml and net.xml");
    try {
        po::store(po::parse_command_line(arg_num, arg_vec, desc), vm);
        po::notify(vm);
    }
    catch (po::error &e) {
        cerr << "ERROR: " << e.what() << endl << endl << desc << endl;
        return 1;
    }
    std::string configFolder = vm["configFolder"].as<std::string>();

    // start simulation
    GlobalResourcesCS& globalResources = GlobalResourcesCS::getInstance();
    std::unique_ptr<NetworkManager> networkManager =
               std::make_unique<NetworkManager>("Layer", configFolder);
    cout << "Random seed " << globalResources.rd_seed << endl;
    cout << endl
         << "Starting Simulation!" << endl;
    sc_start(globalResources.simulation_time, SC_NS);
    cout << endl
         << "Simulation completed! Time: "<< sc_time_stamp() << endl;
    
    TlmReport& report = TlmReport::getInstance();
    report.write_report();
    return 0;
}
