/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once

#ifndef SC_INCLUDE_DYNAMIC_PROCESSES
#define SC_INCLUDE_DYNAMIC_PROCESSES
#endif

#include <systemc>
#include "tlm.h"
#include "string.h"

using namespace sc_dt;

// router paremters
#define NUM_CREDITS     7
// router delays
#define REQ_INIT_DELAY       10
#define REQ_END_DELAY        10
#define INTERN_PROC_DELAY    100
#define RESP_END_DELAY       10
#define UNITS_DELAY          SC_NS

// types
typedef tlm::tlm_generic_payload tlm_gp;

// for circuit switching noc
#define NUM_ACC_LAYERS 2
#define NUM_CREDITS_CS 1
#define TYPE_PACKET "Packet"
#define TYPE_STREAM "Stream"
#define TYPE_ROUT_CONFIG "ConfigRouter"


/********************** EXTENSIONS ****************************/
// Define an extension for the transactions
// link always point to initiator link of transaction
struct link_extension : tlm::tlm_extension<link_extension> {
    int packet_id;
    int flit_id;
    int link;
    int data_type;
    virtual tlm_extension_base* clone() const {
        return new link_extension(*this);
    }
    virtual void copy_from(const tlm_extension_base& ext) {
        packet_id = static_cast<const link_extension&>(ext).packet_id;
        flit_id = static_cast<const link_extension&>(ext).flit_id;
        link = static_cast<const link_extension&>(ext).link;
        data_type = static_cast<const link_extension&>(ext).data_type;
    }
};
