/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#include "noc_logger.h"
#include <iostream>
#include <fstream>
#include <sstream>

#define NOC_LOG "NOC_LOG"

using namespace sc_core;
using namespace sc_dt;
using namespace std;

const string severity_str[5] = {"INFO", "WARNING", "ERROR", "FATAL", ""};

void report_handler(const sc_report& report, const sc_actions& actions){
    stringstream ss_msg;
    ss_msg << report.get_time() << ": " 
           << severity_str[report.get_severity()] << ": "
           << report.get_msg_type() << ": " << report.get_msg() 
           << std::endl;

    if (actions & SC_DISPLAY) {
        string col_code;
        switch (report.get_severity()){
            case SC_INFO:
                col_code = ASCII_GREEN;
                break;
            case SC_WARNING:
                col_code = ASCII_YELLOW;
                break;
            case SC_ERROR:
            case SC_FATAL:
                col_code = ASCII_RED;
                break;
            default:
                col_code = "";
                break;
        }

        if (! report.get_severity()==SC_INFO ){
            string formatted_msg = "\033[0;" + col_code + "m" + 
                                    ss_msg.str() + "\033[0m";
            cout << formatted_msg;
        }
    }

    if (actions & SC_LOG) {
        string log_filename = DEF_LOGFILENAME;
        if(string(report.get_msg_type()) == N_LOG){
            log_filename = N_LOGFILENAME;
        }
        else if(string(report.get_msg_type()) == R_LOG){
            log_filename = R_LOGFILENAME;
        }
        else if(string(report.get_msg_type()) == NI_LOG){
            log_filename = NI_LOGFILENAME;
        }
        else if(string(report.get_msg_type()) == NM_LOG){
            log_filename = NM_LOGFILENAME;
        }
        else if(string(report.get_msg_type()) == PE_LOG){
            log_filename = PE_LOGFILENAME;
        }

        ofstream log_file;
        log_file.open(log_filename, std::ios_base::app);
        log_file << ss_msg.str();
        log_file.close();
    }
}


void setup_logger(){
    // empty old log files
    ofstream log_file;
    log_file.open(N_LOGFILENAME);
    log_file.close();
    log_file.open(R_LOGFILENAME);
    log_file.close();
    log_file.open(NI_LOGFILENAME);
    log_file.close();
    log_file.open(NM_LOGFILENAME);
    log_file.close();
    log_file.open(PE_LOGFILENAME);
    log_file.close();
    log_file.open(DEF_LOGFILENAME);
    log_file.close();
    
    sc_report_handler::set_handler(report_handler);
}

/*void creds_log_info(string router_name, uint8_t link, uint8_t num_creds, 
                                                        bool is_ext){
    string msg = is_ext? ": #external credits" : ": #internal credits";
    SC_REPORT_INFO(NOC_LOG, (router_name + msg + ": on link "+
                    str_dir[link]+": "+to_string(num_creds)).c_str());
}*/


void cout_logger(string msg){
    cout << sc_time_stamp() << ": "<< msg <<endl;
}