/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#include "report.h"
#include <iostream>
#include <fstream>
#include <sstream>

TlmReport::TlmReport(){
}

TlmReport &TlmReport::getInstance()
{
    static TlmReport instance;
    return instance;
}

void TlmReport::update_packet_count(string np_name){
    // check if np_name exists in packet_count
    if(packet_count.count(np_name)){
        packet_count[np_name] += 1;
    }
    else{
        packet_count.insert(std::make_pair(np_name, 1));
    }
}


void TlmReport::update_throttled_count(string np_name){
    if(throttled_count.count(np_name)){
        throttled_count[np_name] += 1;
    }
    else{
        throttled_count.insert(std::make_pair(np_name, 1));
    }
}


void TlmReport::write_report(){
    cout << endl << "Write packet count csv file" << endl;
    stringstream ss_msg_packet;
    ss_msg_packet << "router,packet_count" << endl;
    for(auto& pc_pair: packet_count){
        ss_msg_packet << string(pc_pair.first) << "," << to_string(pc_pair.second) << endl;
    }

    ofstream log_file;
    log_file.open(PACKET_COUNT_FILE);
    log_file << ss_msg_packet.str();
    log_file.close();


    cout << "Write throttled count csv file" << endl << endl;
    stringstream ss_msg_throttled;
    ss_msg_throttled << "router,throttled_count" << endl;
    for(auto& tc_pair: throttled_count){
        ss_msg_throttled << tc_pair.first << "," << tc_pair.second << endl;
    }

    log_file.open(THROTTLED_COUNT_FILE);
    log_file << ss_msg_throttled.str();
    log_file.close();
}
