/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once

#include <map>
//#include <systemc>

#define PACKET_COUNT_FILE "out/packet_count.csv"
#define THROTTLED_COUNT_FILE "out/throttled_count.csv"

using namespace std;
//using namespace sc_core;

class TlmReport {
    public:

        TlmReport();
        static TlmReport &getInstance();

        /**
         * Increases the packet count by one for the given router
         *
         * @param np_name     network participant's name
         */
        void update_packet_count(string np_name);

        /**
         * Increases the throttled count by one for the given router
         *
         * @param np_name     network participant's name
         */
        void update_throttled_count(string np_name);

        /** 
         * Writes report as csv
         */
        void write_report();

    private:
        std::map<string, int> packet_count;
        std::map<string, int> throttled_count;
        //std::map<int, sc_time> task_start_time;
        //std::map<int, sc_time> task_completed_time;
};