/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once

#include "ratatoskrUtils/utils/Structures.h"

struct Message {
	int config_link = 0;
	int dest_link   = 0;
};

struct DataDestinationCS: DataDestination {
    Message msg;

    DataDestinationCS() : DataDestination() {};

    DataDestinationCS(dataDestID_t id, dataTypeID_t dataType, 
        nodeID_t destinationNode, int minInterval, int maxInterval):
            DataDestination(id, dataType, destinationNode, 
            minInterval, maxInterval){};
};

struct DataSendPossibilityCS {
    possID_t id;
    float probability;
    std::vector<DataDestinationCS> dataDestinations;

    DataSendPossibilityCS(possID_t id, float probability, 
        const std::vector<DataDestinationCS>& dataDestinations):
        id(id),
        probability(probability),
        dataDestinations(dataDestinations) {};
};

class TaskCS: public Task{
public:
    std::vector<DataSendPossibilityCS> possibilities;

    TaskCS() : Task() {};

    TaskCS(taskID_t id, nodeID_t nodeID) : Task(id, nodeID) {};

    TaskCS(taskID_t id, nodeID_t nodeID, const std::vector<DataRequirement>& requirements,
            const std::vector<DataSendPossibilityCS>& possibilities) : 
                Task(id, nodeID, requirements, 
                std::vector<DataSendPossibility>()),
                possibilities(possibilities) {};
};