/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#include "utils.h"
#include "configuration.h"
#include "utils/GlobalResourcesCS.h"
#include <fstream>

using namespace std;

void get_pos_distances(float distances[3], vector<float> gr_pos[3]){
    GlobalResourcesCS& globalResources = GlobalResourcesCS::getInstance();
    gr_pos[0] = globalResources.xPositions;
    gr_pos[1] = globalResources.yPositions;
    gr_pos[2] = globalResources.zPositions;
    distances[0] = min_element(gr_pos[0].begin(), gr_pos[0].end())[1];
    distances[1] = min_element(gr_pos[1].begin(), gr_pos[1].end())[1];
    distances[2] = min_element(gr_pos[2].begin(), gr_pos[2].end())[1];
}

void convert_pos_to_int(float pos_float[3], uint8_t pos_int[3]){
    float dists[3];
    vector<float> gr_pos[3];
    get_pos_distances(dists, gr_pos);

    pos_int[0] = round(pos_float[0]/dists[0]);
    pos_int[1] = round(pos_float[1]/dists[1]);
    pos_int[2] = round(pos_float[2]/dists[2]);
}

void get_max_pos(uint8_t max_pos[3]){
    float dists[3];
    vector<float> gr_pos[3];
    get_pos_distances(dists, gr_pos);

    max_pos[0] = dists[0] != 0 ? 
                    1+round(max_element(gr_pos[0].begin(), gr_pos[0].end())[0]/dists[0]) 
                    : 1;
    max_pos[1] = dists[1] != 0 ?
                    1+round(max_element(gr_pos[1].begin(),gr_pos[1].end())[0]/dists[1])
                    : 1;
    max_pos[2] = dists[2] != 0 ?
                    1+round(max_element(gr_pos[2].begin(), gr_pos[2].end())[0]/dists[2])
                    : 1;
}

string get_type_name(dataTypeID_t id){
    GlobalResourcesCS& globalResources = GlobalResourcesCS::getInstance();
    for(int i=0; i < globalResources.dataTypes.size(); i++){
        if(globalResources.dataTypes[i].id == id){
            return globalResources.dataTypes[i].name;
        }
    }
    return "";
}