/*******************************************************************************
 * Copyright (C) 2024 Juan Neyra
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
#pragma once

#include <systemc>
#include "configuration.h"
#include "ratatoskrUtils/utils/Structures.h"

using namespace sc_core;

enum Direction{
    local = 0,
    east,
    west,
    north,
    south,
    up,
    down,
    num_dirs,
    invalid = num_dirs
};

typedef Direction Dir;

/**
 * Get the distance between two elements for every axis
 *
 * @param distances     array to store the value of distances
 */
void get_pos_distances(float distances[3]);

/**
 * Transform the position in range 0 to 1 to an integer value
 * in range 0 to max position-1
 *
 * @param pos_float     array with the positions as float
 * @param pos_float     array to store the value of converted positions
 */
void convert_pos_to_int(float pos_float[3], uint8_t pos_int[3]);

/**
 * Gets the maximum positions of the noc
 *
 * @param max_pos     array to store the value of max positions
 */
void get_max_pos(uint8_t max_pos[3]);

/**
 * Gets the name of type given the id
 *
 * @param id     id type whose name needs to be returned
 */
std::string get_type_name(dataTypeID_t id);